local L = AceLibrary("AceLocale-2.2"):new("Quartz")

local QuartzGCD = Quartz:NewModule('GCD')
local Quartz = Quartz

local GetTime = GetTime

local gcdbar, gcdbar_width, gcdspark, db
local starttime, duration, warned, spell1id, spell2id

local BOOKTYPE_SPELL = BOOKTYPE_SPELL

local function OnUpdate()
	gcdspark:ClearAllPoints()
	local perc = (GetTime() - starttime) / duration
	if perc > 1 then
		return gcdbar:Hide()
	else
		gcdspark:SetPoint('CENTER', gcdbar, 'LEFT', gcdbar_width * perc, 0)
	end
end
local function OnHide()
	gcdbar:SetScript('OnUpdate', nil)
end
local function OnShow()
	gcdbar:SetScript('OnUpdate', OnUpdate)
end

function QuartzGCD:OnInitialize()
	db = Quartz:AcquireDBNamespace("GCD")
	Quartz:RegisterDefaults("GCD", "profile", {
		sparkcolor = {1, 1, 1},
		gcdalpha = 0.9,
		gcdheight = 4,
		gcdposition = L["Bottom"],
	})
	Quartz:RegisterDefaults("GCD", "char", {
		spell = '',
		backupspell = '',
	})
end
function QuartzGCD:OnEnable()
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
	self:RegisterEvent("SPELLS_CHANGED", "ApplySettings")
	if not gcdbar then
		gcdbar = CreateFrame('Frame', 'QuartzGCDBar', UIParent)
		gcdbar:SetFrameStrata('HIGH')
		gcdbar:SetScript('OnShow', OnShow)
		gcdbar:SetScript('OnHide', OnHide)
		
		gcdspark = gcdbar:CreateTexture(nil, 'DIALOG')
		gcdbar:Hide()
	end
	Quartz.ApplySettings()
end
function QuartzGCD:OnDisable()
	gcdbar:Hide()
end

function QuartzGCD:ACTIONBAR_UPDATE_COOLDOWN()
	if spell1id then
		local start, dur = GetSpellCooldown(spell1id, BOOKTYPE_SPELL)
		if dur > 0 and dur <= 2 then
			starttime = start
			duration = dur
			gcdbar:Show()
			return
		end
	end
	if spell2id then
		local start, dur = GetSpellCooldown(spell2id, BOOKTYPE_SPELL)
		if dur > 0 and dur <= 2 then
			starttime = start
			duration = dur
			gcdbar:Show()
			return
		end
	end
end
function QuartzGCD:ApplySettings()
	if gcdbar and Quartz:IsModuleActive('GCD') then
		local ldb = db.profile
		gcdbar:ClearAllPoints()
		gcdbar:SetHeight(ldb.gcdheight)
		gcdbar_width = QuartzCastBar:GetWidth() - 8
		gcdbar:SetWidth(gcdbar_width)
		gcdbar:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16})
		gcdbar:SetBackdropColor(0,0,0)
		gcdbar:SetAlpha(ldb.gcdalpha)
		gcdbar:SetScale(Quartz.db.profile.scale)
		if ldb.gcdposition == L["Bottom"] then
			gcdbar:SetPoint("TOP", QuartzCastBar, "BOTTOM", 0, 2)
		else -- L["Top"]
			gcdbar:SetPoint("BOTTOM", QuartzCastBar, "TOP", 0, 2)
		end
		
		gcdspark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
		gcdspark:SetVertexColor(unpack(ldb.sparkcolor))
		gcdspark:SetBlendMode('ADD')
		gcdspark:SetWidth(15)
		gcdspark:SetHeight(ldb.gcdheight*2.5)
		
		if db.char.spell == '' then
			if not warned then
				Quartz:Print(L["Spell_Warning"])
				warned = true
			end
		else
			spell1id = nil
			spell2id = nil
			for tab = 1, 4 do
				local _, _, offset, numSpells = GetSpellTabInfo(tab)
				for i = (1+offset), (offset+numSpells) do
					local spell = GetSpellName(i, BOOKTYPE_SPELL)
					if spell == db.char.spell then
						spell1id = i
					elseif spell == db.char.backupspell then
						spell2id = i
					end
				end
			end
		end
	end
end
do
	local function set(field, value)
		db.profile[field] = value
		Quartz.ApplySettings()
	end
	local function get(field)
		return db.profile[field]
	end
	local function setcolor(field, ...)
		db.profile[field] = {...}
		Quartz.ApplySettings()
	end
	local function getcolor(field)
		return unpack(db.profile[field])
	end
	local function setspell(field, value)
		for tab = 1, 4 do
			local _, _, offset, numSpells = GetSpellTabInfo(tab)
			for i = (1+offset), (offset+numSpells) do
				local spell = GetSpellName(i, BOOKTYPE_SPELL)
				if spell:lower() == value:lower() then
					db.char[field] = spell
					return
				end
			end
		end
		Quartz:Print(L["Invalid Spell"])
		db.char[field] = ''
		Quartz.ApplySettings()
	end
	local function getspell(field)
		return db.char[field]
	end
	Quartz.options.args.gcd = {
		type = 'group',
		name = L["Global Cooldown"],
		desc = L["Global Cooldown"],
		order = 601,
		args = {
			toggle = {
				type = 'toggle',
				name = L["Enable"],
				desc = L["Enable"],
				get = function()
					return Quartz:IsModuleActive('GCD')
				end,
				set = function(v)
					Quartz:ToggleModuleActive('GCD', v)
				end,
				order = 100,
			},
			spell = {
				type = 'text',
				name = L["Spell"],
				desc = L["%s spell to check for the Global Cooldown"]:format(L["Primary"]),
				get = getspell,
				set = setspell,
				passValue = 'spell',
				usage = L["<Spell Name>"],
				order = 101,
			},
			backupspell = {
				type = 'text',
				name = L["Spell"],
				desc = L["%s spell to check for the Global Cooldown"]:format(L["Primary"]),
				get = getspell,
				set = setspell,
				passValue = 'backupspell',
				usage = L["<Spell Name>"],
				order = 101,
			},
			gcdcolor = {
				type = 'color',
				name = L["Spark Color"],
				desc = L["Spark Color"],
				get = getcolor,
				set = setcolor,
				passValue = 'sparkcolor',
				order = 103,
			},
			gcdheight = {
				type = 'range',
				name = L["Height"],
				desc = L["Height"],
				min = 2,
				max = 10,
				step = 1,
				get = get,
				set = set,
				passValue = 'gcdheight',
				order = 104,
			},
			gcdalpha = {
				type = 'range',
				name = L["Alpha"],
				desc = L["Alpha"],
				min = 0.05,
				max = 1,
				step = 0.05,
				isPercent = true,
				get = get,
				set = set,
				passValue = 'gcdalpha',
				order = 105,
			},
			gcdposition = {
				type = 'text',
				name = L["Bar Position"],
				desc = L["Bar Position"],
				get = get,
				set = set,
				passValue = 'gcdposition',
				validate = {L["Top"], L["Bottom"]},
				order = 106,
			},
		},
	}
end